﻿namespace SerialPWMController
{



    #region Using Statements:



    using System;
    using System.IO.Ports;
    using System.Windows.Forms;



    #endregion




    public partial class Form1 : Form
    {



        #region Fields:



        /// <summary>
        /// The Microcontroller Clock Frequency:
        /// </summary>
        int fclock = 16000000;



        /// <summary>
        /// The Clock Frequency Prescaler:
        /// </summary>
        int Prescaler = 1024;



        /// <summary>
        /// Create a new SerialPort Reference.
        /// <para>
        ///     See: https://docs.microsoft.com/en-us/dotnet/api/system.io.ports.serialport?view=dotnet-plat-ext-3.1
        /// </para>
        /// </summary>
        SerialPort serialPort = new SerialPort();



        #endregion



        #region Properties:



        /// <summary>
        /// The System Frequency.
        /// </summary>
        public double Frequency { get; set; } = 1.00;



        /// <summary>
        /// The System Duty Cycle.
        /// </summary>
        public double DutyCycle { get; set; } = 50.00;



        /// <summary>
        /// The Frequency Track Bar Multiplier.
        /// </summary>
        private int Multiplier { get; set; } = 1;



        /// <summary>
        /// Serial Port Name:
        /// </summary>
        public string PortName
        {
            get { return serialPort.PortName; }
            internal set { serialPort.PortName = value; }
        }



        /// <summary>
        /// Serial Port BaudRate:
        /// </summary>
        public int BaudRate
        {
            get { return serialPort.BaudRate; }
            internal set { serialPort.BaudRate = value; }
        }



        /// <summary>
        /// Serial Port Parity:
        /// </summary>
        public Parity Parity
        {
            get { return serialPort.Parity; }
            internal set { serialPort.Parity = value; }
        }



        /// <summary>
        /// Serial Port DataBits:
        /// </summary>
        public int DataBits
        {
            get { return serialPort.DataBits; }
            internal set { serialPort.DataBits = value; }
        }



        /// <summary>
        /// Serial Port StopBits:
        /// </summary>
        public StopBits StopBits
        {
            get { return serialPort.StopBits; }
            internal set { serialPort.StopBits = value; }
        }



        /// <summary>
        /// Serial Port Handshake:
        /// </summary>
        public Handshake Handshake
        {
            get { return serialPort.Handshake; }
            internal set { serialPort.Handshake = value; }
        }



        #endregion



        /// <summary>
        /// Form Constructor.
        /// </summary>
        public Form1()
        {

            // Form Init:
            InitializeComponent();

            // Add Port Names:
            foreach (string PortName in SerialPort.GetPortNames())
                if (!PortNameComboBox.Items.Contains(PortName))
                    PortNameComboBox.Items.Add(PortName);

            // Standard Baud Rates:
            int[] BaudRates = new int[] { 110, 300, 600, 1200, 2400, 4800, 9600, 14400, 19200, 38400, 57600, 115200, 128000, 256000 };

            // Add Port Names:
            foreach (int BaudRate in BaudRates)
                BaudRateComboBox.Items.Add(BaudRate);

            // Set Selected Index:
            BaudRateComboBox.SelectedItem = 115200;

            // Attach Frequency TrackBar Value Changed Event Handler:
            this.FrequencyTrackBar.ValueChanged += new System.EventHandler(this.FrequencyTrackBar_ValueChanged);
        }



        /// <summary>
        /// Form Load Event.
        /// </summary>
        private void Form1_Load(object sender, EventArgs e)
        {

            // Set Initial Frequency:
            FrequencyTextBox.Text = "1111.111";

            // Set Initial Duty Cycle:
            DutyCycleTextBox.Text = DutyCycle.ToString();

            // Set Duty Cycle Track Bar:
            DutyCycleTrackBar.Value = (int)(50.0 * 1000.00);
        }



        /// <summary>
        /// Calculate the Frequency:
        /// </summary>
        /// <param name="frequency">The desired Frequency: 3.73Hz</param>
        /// <returns>int representing the resolution on the Register (16 bits).</returns>
        int CalculateFrequency(double frequency)
        {

            // Calculate OCR1A from specified Frequency:
            return (int)(Convert.ToDouble(fclock) / (frequency * Convert.ToDouble(Prescaler)) - 1.0);
        }



        /// <summary>
        /// Calculate the Duty Cycle:
        /// </summary>
        /// <param name="period">Period, or 1 / F</param>
        /// <param name="duty">The desired Duty Cycle: 50.0%</param>
        /// <returns>int representing the resolution on the Register (16 bits).</returns>
        int CalculateDutyCycle(int period, double duty)
        {

            // Calculate OCR1B from OCR1A and specified Duty Cycle:
            return (short)(Convert.ToDouble(period) * (duty / 100.00));
        }



        /// <summary>
        /// Calculate Register Resolution:
        /// </summary>
        /// <param name="ocrx">The Register Value, E.G: CalculateFrequency(1024.57)</param>
        /// <returns>double representing the Resolution at the current Register Value.</returns>
        double CalculateResolution(int ocrx)
        {

            return (Math.Log((double)ocrx + 1.0D) / Math.Log(2.0D));
        }



        /// <summary>
        /// Send Button Click Event.
        /// </summary>
        private void SendButton_Click(object sender, EventArgs e)
        {

            // Parse the Frequency Value:
            if (!double.TryParse(FrequencyTextBox.Text, out double Frequency))
            {

                // Show Error:
                richTextBox1.AppendText("Input Error!" + Environment.NewLine);

                // Return:
                return;
            }

            // Parse the Duty Cycle Value:
            if (!double.TryParse(DutyCycleTextBox.Text, out double DutyCycle))
            {

                // Show Error:
                richTextBox1.AppendText("Input Error!" + Environment.NewLine);

                // Return:
                return;
            }

            // Check Values:
            if (Frequency == 0.00 || DutyCycle == 0.00)
            {

                // Show Error:
                richTextBox1.AppendText("Error!\r\nCant Divide by Zero!" + Environment.NewLine);

                // Return:
                return;
            }

            // Check Values:
            if (Frequency < 0.00 || DutyCycle < 0.00)
            {

                // Show Error:
                richTextBox1.AppendText("Error!\r\nOnly Positive Numbers can be used!" + Environment.NewLine);

                // Return:
                return;
            }

            // Calculate Prescaler:
            Prescaler = CalculatePrescaler(Frequency);

            // Calculate Resolutions:
            int FrequencyResolution = CalculateFrequency(Frequency);
            int DutyCycleResolution = CalculateDutyCycle(FrequencyResolution, DutyCycle);

            // Set Resolution:
            ResolutionLabel.Text = CalculateResolution(FrequencyResolution).ToString();

            try
            {

                // Send the Data:
                serialPort.WriteLine(String.Format("<{0}|{1}|{2}>", FrequencyResolution, DutyCycleResolution, Prescaler));
            }
            catch
            {

                // Show Error:
                richTextBox1.AppendText("Error!\r\nSerial is not Connected, please connect and try again!" + Environment.NewLine);

                // Return:
                return;
            }
        }



        /// <summary>
        /// Calculate the Prescaler.
        /// </summary>
        /// <param name="frequency"></param>
        /// <returns></returns>
        private int CalculatePrescaler(double frequency)
        {

            if (frequency >= 300000.00)
                return 1;

            if (frequency <= 300000.00 && frequency >= 10000.00)
                return 8;

            if (frequency < 10000.00 && frequency >= 3000.00)
                return 64;

            if (frequency < 3000.00 && frequency >= 500.00)
                return 256;

            if (frequency < 500.00 && frequency >= 0.00)
                return 1024;

            return 0;
        }



        /// <summary>
        /// Serial Data Received Event.
        /// </summary>
        private void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            // Handle Data here...
            string data = ((SerialPort)sender).ReadExisting();
            Invoke(new Action(() =>
            {
                richTextBox1.AppendText(data + Environment.NewLine);
                richTextBox1.ScrollToCaret();
            }));
        }



        /// <summary>
        /// Connect Button Click Event.
        /// </summary>
        private void ConnectButton_Click(object sender, EventArgs e)
        {

            if (ConnectButton.Text == "Connect")
            {

                // RichTextBox Clear:
                richTextBox1.Clear();

                // Configure Serial Properties:
                string Port = PortNameComboBox.SelectedItem.ToString();
                int Baud = Convert.ToInt32(BaudRateComboBox.SelectedItem.ToString());
                serialPort = new SerialPort(Port);
                BaudRate = Baud;
                Parity = Parity.None;
                DataBits = 8;
                StopBits = StopBits.One;
                Handshake = Handshake.None;

                // Gets or sets a value indicating whether the Request to Send (RTS) signal is enabled during serial communication.
                serialPort.RtsEnable = true;

                // Attach Data Reveived Event:
                serialPort.DataReceived += SerialPort_DataReceived;

                try
                {
                    // Open the Serial Port:
                    serialPort.Open();

                    // Change Button Text:
                    ConnectButton.Text = "Disconnect";

                    // Return:
                    return;
                }
                catch (Exception exc)
                {

                    // Show Error:
                    richTextBox1.AppendText(exc.Message + Environment.NewLine);

                    // Return:
                    return;
                }
            }

            if (ConnectButton.Text == "Disconnect")
            {

                // Close the Serial Port:
                serialPort.Close();
                serialPort.Dispose();

                // Change Button Text:
                ConnectButton.Text = "Connect";

                // Return:
                return;
            }
        }



        /// <summary>
        /// The Frequency TrackBar Value Changed Event.
        /// </summary>
        private void FrequencyTrackBar_ValueChanged(object sender, EventArgs e)
        {

            // Detach Frequency TrackBar Value Changed Event Handler:
            FrequencyTrackBar.ValueChanged -= new System.EventHandler(FrequencyTrackBar_ValueChanged);

            // Get TrackBar Frequency Value:
            int Value = (sender as TrackBar).Value;

            // Check Frequency Values:
            if (Value >= FrequencyTrackBar.Maximum)
            {

                // Increment the Multiplier:
                Multiplier = Multiplier + 100;

                // Check for Ones Value:
                if (Multiplier == 101)
                {
                    Multiplier = 100;
                    Frequency = 1.00;
                }

                // Reset Track Bar after Value Underflow:
                FrequencyTrackBar.Value = FrequencyTrackBar.Minimum + 1;
            }
            else
            if (Value <= FrequencyTrackBar.Minimum)
            {

                // Decrement the Multiplier:
                Multiplier = Multiplier - 100;

                // Reset Multiplier:
                if (Multiplier <= 0)
                {
                    Multiplier = 1;
                    Frequency = 0.001;
                }

                // Check Zero Values:
                if (Frequency >= 1)
                    // Reset Track Bar after Value Overflow:
                    FrequencyTrackBar.Value = FrequencyTrackBar.Maximum - 1;
            }

            // Reattach Frequency TrackBar Value Changed Event Handler:
            FrequencyTrackBar.ValueChanged += new System.EventHandler(FrequencyTrackBar_ValueChanged);

            // Set the Frequency:
            // Track Bar Max Value is: 100000 so / 1000 gives increments of 100, with 3 Decimal places: 100.123
            FrequencyTextBox.Text = ((Frequency * Convert.ToDouble(Multiplier)) + (Convert.ToDouble(Value) / 1000.0D)).ToString();
        }



        /// <summary>
        /// The Duty Cycle TrackBar Value Changed Event.
        /// </summary>
        private void DutyCycleTrackBar_ValueChanged(object sender, EventArgs e)
        {

            // Calculate Duty Cycle:
            DutyCycle = Math.Round(DutyCycleTrackBar.Value / 1000.00D, 3);

            // Set the Duty Cycle Text Box:
            this.DutyCycleTextBox.Text = DutyCycle.ToString();
        }



        /// <summary>
        /// Frequency TextBox Leave Event.
        /// </summary>
        private void FrequencyTextBox_Leave(object sender, EventArgs e)
        {

            // Get the Frequency:
            double.TryParse(FrequencyTextBox.Text, out double FrequencyOut);

            // Get Hundreds:
            int Hundreds = Convert.ToInt32(Math.Floor(FrequencyOut / 100));

            // Check Multiplier:
            if (Hundreds <= 0.00 && FrequencyOut < 100.00)
                // Set Multiplier:
                Multiplier = 1;
            else
                if (Hundreds <= 0.00 && FrequencyOut > 100.00)
                // Set Multiplier:
                Multiplier = 100 * 1;
            else
                // Set Multiplier:
                Multiplier = 100 * Hundreds;

            // Detach Frequency TrackBar Value Changed Event Handler:
            FrequencyTrackBar.ValueChanged -= new System.EventHandler(FrequencyTrackBar_ValueChanged);

            // Calculate the Frequency Divider, including the 3 Decimal places:
            double FrequencyDivider = Convert.ToInt32(FrequencyOut * 1000);

            // Set the Frequency TrackBar Value:
            if (FrequencyDivider < FrequencyTrackBar.Maximum)
                FrequencyTrackBar.Value = Convert.ToInt32(FrequencyDivider);
            else
            {
                FrequencyTrackBar.Value = Convert.ToInt32(FrequencyDivider % FrequencyTrackBar.Maximum);
            }

            // Reattach Frequency TrackBar Value Changed Event Handler:
            FrequencyTrackBar.ValueChanged += new System.EventHandler(FrequencyTrackBar_ValueChanged);
        }
    }
}